"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const bson_objectid_1 = __importDefault(require("bson-objectid"));
const activity_service_1 = require("./activity.service");
const actor_entity_1 = require("./actor.entity");
const assert_1 = __importDefault(require("assert"));
const uri_object_1 = require("./uri.object");
describe('ActivityService', function () {
    describe('#createArticleForPost', function () {
        it('Adds a Create activity for an Article Object to the default actors Outbox', async function () {
            const actor = actor_entity_1.Actor.create({ username: 'testing' });
            const mockActorRepository = {
                async getOne() {
                    return actor;
                },
                async save() { }
            };
            const mockPostRepository = {
                async getOne(id) {
                    return {
                        id: id,
                        title: 'Testing',
                        slug: 'testing',
                        html: '<p> Testing stuff.. </p>',
                        visibility: 'public',
                        authors: ['Mr Bean'],
                        publishedAt: new Date(),
                        featuredImage: null,
                        excerpt: 'Small text',
                        url: new uri_object_1.URI('blah')
                    };
                }
            };
            const service = new activity_service_1.ActivityService(mockActorRepository, mockPostRepository);
            const postId = new bson_objectid_1.default();
            await service.createArticleForPost(postId);
            const found = actor.outbox.find(activity => activity.type === 'Create');
            assert_1.default.ok(found);
        });
        it('Does not add a Create activity for non public posts', async function () {
            const actor = actor_entity_1.Actor.create({ username: 'testing' });
            const mockActorRepository = {
                async getOne() {
                    return actor;
                },
                async save() { }
            };
            const mockPostRepository = {
                async getOne(id) {
                    return {
                        id: id,
                        title: 'Testing',
                        slug: 'testing',
                        html: '<p> Testing stuff.. </p>',
                        visibility: 'private',
                        authors: ['Mr Bean'],
                        publishedAt: new Date(),
                        featuredImage: null,
                        excerpt: 'Small text',
                        url: new uri_object_1.URI('blah')
                    };
                }
            };
            const service = new activity_service_1.ActivityService(mockActorRepository, mockPostRepository);
            const postId = new bson_objectid_1.default();
            await service.createArticleForPost(postId);
            const found = actor.outbox.find(activity => activity.type === 'Create');
            assert_1.default.ok(!found);
        });
        it('Throws when post is not found', async function () {
            const actor = actor_entity_1.Actor.create({ username: 'testing' });
            const mockActorRepository = {
                async getOne() {
                    return actor;
                },
                async save() { }
            };
            const mockPostRepository = {
                async getOne() {
                    return null;
                }
            };
            const service = new activity_service_1.ActivityService(mockActorRepository, mockPostRepository);
            const postId = new bson_objectid_1.default();
            await assert_1.default.rejects(async () => {
                await service.createArticleForPost(postId);
            }, /Post not found/);
        });
        it('Throws when actor is not found', async function () {
            const mockActorRepository = {
                async getOne() {
                    return null;
                },
                async save() { }
            };
            const mockPostRepository = {
                async getOne(id) {
                    return {
                        id: id,
                        title: 'Testing',
                        slug: 'testing',
                        html: '<p> Testing stuff.. </p>',
                        visibility: 'private',
                        authors: ['Mr Bean'],
                        publishedAt: new Date(),
                        featuredImage: null,
                        excerpt: 'Small text',
                        url: new uri_object_1.URI('blah')
                    };
                }
            };
            const service = new activity_service_1.ActivityService(mockActorRepository, mockPostRepository);
            const postId = new bson_objectid_1.default();
            await assert_1.default.rejects(async () => {
                await service.createArticleForPost(postId);
            }, /Actor not found/);
        });
    });
});
//# sourceMappingURL=activity.service.test.js.map