"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_crypto_1 = __importDefault(require("node:crypto"));
const actor_entity_1 = require("./actor.entity");
const http_signature_service_1 = require("./http-signature.service");
const node_assert_1 = __importDefault(require("node:assert"));
const uri_object_1 = require("./uri.object");
const activity_entity_1 = require("./activity.entity");
const article_object_1 = require("./article.object");
const bson_objectid_1 = __importDefault(require("bson-objectid"));
describe('Actor', function () {
    describe('getters', function () {
        describe('displayName', function () {
            it('Uses displayName, but falls back to username', function () {
                const hasDisplayName = actor_entity_1.Actor.create({
                    username: 'username',
                    displayName: 'displayName'
                });
                const doesnaeHaveDisplayName = actor_entity_1.Actor.create({
                    username: 'username'
                });
                node_assert_1.default.equal(hasDisplayName.displayName, 'displayName');
                node_assert_1.default.equal(doesnaeHaveDisplayName.displayName, 'username');
            });
        });
        describe('actorId', function () {
            it('Correctly returns the actor url', function () {
                const actor = actor_entity_1.Actor.create({ username: 'testing' });
                const idString = actor.id.toHexString();
                const actorId = actor.actorId;
                const baseUrl = new URL('https://domain.tld/base');
                node_assert_1.default.equal(actorId.getValue(baseUrl), `https://domain.tld/base/actor/${idString}`);
            });
        });
    });
    describe('#createArticle', function () {
        it('Adds an activity to the outbox', function () {
            const actor = actor_entity_1.Actor.create({ username: 'username' });
            const article = article_object_1.Article.fromPost({
                id: new bson_objectid_1.default(),
                title: 'Post Title',
                slug: 'post-slug',
                html: '<p>Hello world</p>',
                visibility: 'public',
                url: new uri_object_1.URI(''),
                authors: ['Mr Burns'],
                featuredImage: null,
                publishedAt: null,
                excerpt: 'Hey'
            });
            actor.createArticle(article);
            const found = actor.outbox.find((value) => {
                return value.type === 'Create';
            });
            node_assert_1.default.ok(found);
        });
    });
    describe('#sign', function () {
        it('returns a request with a valid Signature header', async function () {
            const baseUrl = new URL('https://example.com/ap');
            const actor = actor_entity_1.Actor.create({
                username: 'Testing'
            });
            const url = new URL('https://some-server.com/users/username/inbox');
            const date = new Date();
            const request = new Request(url, {
                headers: {
                    Host: url.host,
                    Date: date.toISOString(),
                    Accept: 'application/ld+json; profile="https://www.w3.org/ns/activitystreams"'
                }
            });
            const signedRequest = await actor.sign(request, baseUrl);
            const publicKey = actor.getJSONLD(baseUrl).publicKey;
            class MockHTTPSignature extends http_signature_service_1.HTTPSignature {
                static async getPublicKey() {
                    return node_crypto_1.default.createPublicKey(publicKey.publicKeyPem);
                }
            }
            const signedRequestURL = new URL(signedRequest.url);
            const actual = await MockHTTPSignature.validate(signedRequest.method, signedRequestURL.pathname, signedRequest.headers);
            const expected = true;
            node_assert_1.default.equal(actual, expected, 'The signature should have been valid');
        });
    });
    describe('#follow', function () {
        it('Creates a Follow activity', async function () {
            const actor = actor_entity_1.Actor.create({ username: 'TestingFollow' });
            const actorToFollow = {
                id: new uri_object_1.URI('https://activitypub.server/actor'),
                username: '@user@domain'
            };
            actor.follow(actorToFollow);
            actor_entity_1.Actor.getActivitiesToSave(actor, function (activities) {
                const followActivity = activities.find(activity => activity.type === 'Follow');
                node_assert_1.default.equal(followActivity?.objectId.href, actorToFollow.id.href);
            });
            actor_entity_1.Actor.getEventsToDispatch(actor, function (events) {
                const followActivityEvent = events.find(event => event.data.activity?.type === 'Follow');
                node_assert_1.default.equal(followActivityEvent.data.activity.objectId.href, actorToFollow.id.href);
            });
        });
    });
    describe('#postToInbox', function () {
        it('Handles Follow activities', async function () {
            const actor = actor_entity_1.Actor.create({ username: 'TestingPostToInbox' });
            const newFollower = new uri_object_1.URI('https://activitypub.server/actor');
            const followActivity = new activity_entity_1.Activity({
                activity: new uri_object_1.URI(`https://activitypub.server/activity`),
                type: 'Follow',
                actor: {
                    id: newFollower,
                    type: 'Person'
                },
                object: {
                    id: actor.actorId,
                    type: 'Person'
                },
                to: actor.actorId
            });
            await actor.postToInbox(followActivity);
            (0, node_assert_1.default)(actor.followers.find(follower => follower.id.href === newFollower.href));
        });
        it('Throws if the Follow activity is anonymous', async function () {
            const actor = actor_entity_1.Actor.create({ username: 'TestingPostToInbox' });
            const newFollower = new uri_object_1.URI('https://activitypub.server/actor');
            const followActivity = new activity_entity_1.Activity({
                activity: null,
                type: 'Follow',
                actor: {
                    id: newFollower,
                    type: 'Person'
                },
                object: {
                    id: actor.actorId,
                    type: 'Person'
                },
                to: actor.actorId
            });
            let error = null;
            try {
                await actor.postToInbox(followActivity);
            }
            catch (err) {
                error = err;
            }
            node_assert_1.default.ok(error);
        });
        it('Handles Accept activities', async function () {
            const actor = actor_entity_1.Actor.create({ username: 'TestingPostToInbox' });
            const newFollower = new uri_object_1.URI('https://activitypub.server/actor');
            const activity = new activity_entity_1.Activity({
                activity: null,
                type: 'Accept',
                actor: {
                    id: newFollower,
                    type: 'Person'
                },
                object: {
                    id: newFollower,
                    type: 'Person'
                },
                to: actor.actorId
            });
            await actor.postToInbox(activity);
            (0, node_assert_1.default)(actor.following.find(follower => follower.id.href === newFollower.href));
            (0, node_assert_1.default)(actor.following.find(follower => follower.username === '@index@activitypub.server'));
        });
    });
    describe('#toJSONLD', function () {
        it('Returns valid JSONLD', async function () {
            const actor = actor_entity_1.Actor.create({ username: 'TestingJSONLD' });
            const baseUrl = new URL('https://example.com');
            const jsonld = actor.getJSONLD(baseUrl);
            node_assert_1.default.ok(jsonld);
        });
    });
});
//# sourceMappingURL=actor.entity.test.js.map