"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const sinon_1 = __importDefault(require("sinon"));
const actor_entity_1 = require("./actor.entity");
const jsonld_service_1 = require("./jsonld.service");
const assert_1 = __importDefault(require("assert"));
const bson_objectid_1 = __importDefault(require("bson-objectid"));
describe('JSONLDService', function () {
    describe('#getActor', function () {
        it('Returns JSONLD representation of Actor', async function () {
            const actor = actor_entity_1.Actor.create({ username: 'freddy' });
            const mockActorRepository = {
                getOne: sinon_1.default.stub().resolves(actor),
                save: sinon_1.default.stub().rejects()
            };
            const mockPostRepository = {
                getOne: sinon_1.default.stub().resolves(null)
            };
            const url = new URL('https://example.com');
            const service = new jsonld_service_1.JSONLDService(mockActorRepository, mockPostRepository, url);
            const result = await service.getActor(actor.id);
            (0, assert_1.default)(result);
            assert_1.default.equal(result.type, 'Person');
        });
    });
    describe('#getOutbox', function () {
        it('returns JSONLD representation of Outbox', async function () {
            const actor = actor_entity_1.Actor.create({ username: 'freddy' });
            const mockActorRepository = {
                getOne: sinon_1.default.stub().resolves(actor),
                save: sinon_1.default.stub().rejects()
            };
            const mockPostRepository = {
                getOne: sinon_1.default.stub().resolves(null)
            };
            const url = new URL('https://example.com');
            const service = new jsonld_service_1.JSONLDService(mockActorRepository, mockPostRepository, url);
            const result = await service.getOutbox(actor.id);
            (0, assert_1.default)(result);
            assert_1.default.equal(result.type, 'OrderedCollection');
        });
        it('returns null if actor not found', async function () {
            const actor = actor_entity_1.Actor.create({ username: 'freddy' });
            const mockActorRepository = {
                getOne: sinon_1.default.stub().resolves(null),
                save: sinon_1.default.stub().rejects()
            };
            const mockPostRepository = {
                getOne: sinon_1.default.stub().resolves(null)
            };
            const url = new URL('https://example.com');
            const service = new jsonld_service_1.JSONLDService(mockActorRepository, mockPostRepository, url);
            const result = await service.getOutbox(actor.id);
            assert_1.default.equal(result, null);
        });
    });
    describe('#getArticle', function () {
        it('Throws if no post found', async function () {
            const mockActorRepository = {
                getOne: sinon_1.default.stub().resolves(null),
                save: sinon_1.default.stub().rejects()
            };
            const mockPostRepository = {
                getOne: sinon_1.default.stub().resolves(null)
            };
            const url = new URL('https://example.com');
            const service = new jsonld_service_1.JSONLDService(mockActorRepository, mockPostRepository, url);
            await assert_1.default.rejects(async () => {
                await service.getArticle(new bson_objectid_1.default());
            });
        });
        it('Throws if post not public', async function () {
            const mockActorRepository = {
                getOne: sinon_1.default.stub().resolves(null),
                save: sinon_1.default.stub().rejects()
            };
            const mockPostRepository = {
                getOne: sinon_1.default.stub().resolves({
                    visibility: 'private'
                })
            };
            const url = new URL('https://example.com');
            const service = new jsonld_service_1.JSONLDService(mockActorRepository, mockPostRepository, url);
            await assert_1.default.rejects(async () => {
                await service.getArticle(new bson_objectid_1.default());
            });
        });
    });
});
//# sourceMappingURL=jsonld.service.test.js.map